/*------------------------------------------------------------------------------
--       Copyright (c) 2015-2017, VeriSilicon Inc. All rights reserved        --
--         Copyright (c) 2011-2014, Google Inc. All rights reserved.          --
--                                                                            --
-- This software is confidential and proprietary and may be used only as      --
--   expressly authorized by VeriSilicon in a written licensing agreement.    --
--                                                                            --
--         This entire notice must be reproduced on all copies                --
--                       and may not be removed.                              --
--                                                                            --
--------------------------------------------------------------------------------
-- Redistribution and use in source and binary forms, with or without         --
-- modification, are permitted provided that the following conditions are met:--
--   * Redistributions of source code must retain the above copyright notice, --
--       this list of conditions and the following disclaimer.                --
--   * Redistributions in binary form must reproduce the above copyright      --
--       notice, this list of conditions and the following disclaimer in the  --
--       documentation and/or other materials provided with the distribution. --
--   * Neither the names of Google nor the names of its contributors may be   --
--       used to endorse or promote products derived from this software       --
--       without specific prior written permission.                           --
--------------------------------------------------------------------------------
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"--
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE  --
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE --
-- ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE  --
-- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR        --
-- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF       --
-- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS   --
-- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN    --
-- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)    --
-- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE --
-- POSSIBILITY OF SUCH DAMAGE.                                                --
--------------------------------------------------------------------------------
------------------------------------------------------------------------------*/

#include "basetype.h"
#include "dwl_hw_core_array.h"
#include "dwl_swhw_sync.h"

#include <assert.h>
#include <errno.h>
#include <fcntl.h>
#include <pthread.h>
#include <semaphore.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/syscall.h>
#include <sys/types.h>
#include <unistd.h>

#ifdef _DWL_DEBUG
#define DWL_DEBUG(fmt, args...) \
  fprintf(stderr, __FILE__ ":%d:%s() " fmt, __LINE__, __func__, ##args)
#else
#define DWL_DEBUG(fmt, args...) \
  do {                          \
  } while (0)
#endif

extern HwCoreArray g_hw_core_array;

void *ThreadMcListener(void *args) {
  struct McListenerThreadParams *params = (struct McListenerThreadParams *)args;

  while (!params->b_stopped) {
    u32 i, ret;

    ret = WaitAnyCoreRdy(g_hw_core_array);
    assert(ret == 0);
    (void)ret;

    if (params->b_stopped) break;

    /* check all decoder IRQ status register */
    for (i = 0; i < params->n_dec_cores; i++) {
      Core c = GetCoreById(g_hw_core_array, i);

      /* check DEC IRQ status */
      if (HwCoreIsDecRdy(c)) {
        DWL_DEBUG("DEC IRQ by Core %d\n", i);

        if (params->callback[i] != NULL) {
          params->callback[i](params->callback_arg[i], i);
        } else {
          /* single Core instance => post dec ready */
          HwCorePostDecRdy(c);
        }

        break;
      }
    }

    /* check all post-processor IRQ status register */
    for (i = 0; i < params->n_ppcores; i++) {
      Core c = GetCoreById(g_hw_core_array, i);

      /* check PP IRQ status */
      if (HwCoreIsPpRdy(c)) {
        DWL_DEBUG("PP IRQ by Core %d\n", i);

        /* single Core instance => post pp ready */
        HwCorePostPpRdy(c);

        break;
      }
    }
  }

  return NULL;
}
