/*------------------------------------------------------------------------------
--       Copyright (c) 2015-2017, VeriSilicon Inc. All rights reserved        --
--         Copyright (c) 2011-2014, Google Inc. All rights reserved.          --
--         Copyright (c) 2007-2010, Hantro OY. All rights reserved.           --
--                                                                            --
-- This software is confidential and proprietary and may be used only as      --
--   expressly authorized by VeriSilicon in a written licensing agreement.    --
--                                                                            --
--         This entire notice must be reproduced on all copies                --
--                       and may not be removed.                              --
--                                                                            --
--------------------------------------------------------------------------------
-- Redistribution and use in source and binary forms, with or without         --
-- modification, are permitted provided that the following conditions are met:--
--   * Redistributions of source code must retain the above copyright notice, --
--       this list of conditions and the following disclaimer.                --
--   * Redistributions in binary form must reproduce the above copyright      --
--       notice, this list of conditions and the following disclaimer in the  --
--       documentation and/or other materials provided with the distribution. --
--   * Neither the names of Google nor the names of its contributors may be   --
--       used to endorse or promote products derived from this software       --
--       without specific prior written permission.                           --
--------------------------------------------------------------------------------
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"--
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE  --
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE --
-- ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE  --
-- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR        --
-- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF       --
-- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS   --
-- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN    --
-- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)    --
-- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE --
-- POSSIBILITY OF SUCH DAMAGE.                                                --
--------------------------------------------------------------------------------
--                                                                            --
-  Description : ...
-
------------------------------------------------------------------------------*/
#ifndef __VIDSTABCOMMON_H__
#define __VIDSTABCOMMON_H__

#include "basetype.h"

#ifdef H8290_HAVE_VIDSTABDEBUG_H
#include "vidstabdebug.h"
#else
#ifndef ASSERT
#define ASSERT(expr)
#endif
#ifndef DEBUG_PRINT
#define DEBUG_PRINT(args)
#endif
#endif

typedef struct HWStabData_
{
    u32 rMotionSum;
    u32 rMotionMin;
    i32 rGmvX;
    i32 rGmvY;
    u32 rMatrixVal[9];
} HWStabData;

/* Stabilizer parameter and data structure */
typedef struct SwStbData_
{
    /* User defined parameters */
    i32 inputWidth; /* Input picture dimensions in pixels */
    i32 inputHeight;
    i32 stride; /* Input picture scanline length in bytes */
    i32 stabilizedWidth;    /* Stabilized picture dimensions in pixels, */
    i32 stabilizedHeight;   /*   multiple of 4, [8, inputHeight-8] */

    /* Stabilized picture position from top-left corner of input picture,
     * algorithm input/output, initialize to (inputWidth-stabilizedWidth)/2 */
    i32 stabOffsetX;    /* [0, inputWidth-stabilizedWidth] */
    i32 stabOffsetY;    /* [0, inputHeight-stabilizedHeight] */

    /* Algorithm internal data, MUST INITIALIZE TO ZERO */
    i32 qpMotionX;
    i32 qpMotionY;
    i32 filterPosX;
    i32 filterPosY;
    i32 filterX;
    i32 filterY;
    i32 filterLengthX;
    i32 filterLengthY;
    i32 filterSumMotX;
    i32 filterSumMotY;
    i32 filterSumStabX;
    i32 filterSumStabY;
    i32 filterErrorX;
    i32 filterErrorY;
    i32 filterErrorXp;
    i32 filterErrorYp;
    const u32 *motion;
    i32 half[5][5];
    i32 quarter[5][5];
    u32 prevMin;
    u32 prevMean;
    u32 sceneChange;
} SwStbData;

void VSAlgInit(SwStbData * data, u32 srcWidth, u32 srcHeight, u32 width,
               u32 height);
void VSAlgReset(SwStbData * data);
u32 VSAlgStabilize(SwStbData * data, const HWStabData * hwStabData);
void VSAlgGetResult(const SwStbData * data, u32 * xOff, u32 * yOff);
void VSReadStabData(const u32 * regMirror, HWStabData * hwStabData);

#endif /* __VIDSTABCOMMON_H__ */
